/*
 * Copyright (C) 2004-2010 Geometer Plus <contact@geometerplus.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#include <windows.h>

#include "ZLWin32RootDir.h"

void ZLWin32RootDir::collectSubDirs(std::vector<std::string> &names, bool) {
	char *buffer = new char[26 * 4];
	DWORD length = GetLogicalDriveStringsA(26 * 4, buffer);
	for (char *ptr = buffer; ptr < buffer + length; ) {
		std::string drive = ptr;
		names.push_back(drive);
		ptr += drive.length() + 1;
	}
	delete[] buffer;
}

void ZLWin32RootDir::collectFiles(std::vector<std::string>&, bool) {
}
